/*****************************************************************************************
*                                                                                        *
**PROJECT:   Robust versus consistent variance estimators in marginal					 *
*			 structural Cox models		 												 *
*                                                                                        *
**PROGRAM NAME: 1_sim_data1.sas	                                                         *
*                                                                                        *
**AUTHOR:       D. Enders					                                     	     *
*                                                                                        *
**VERSION/DATE: V 1.0 / Nov 13th 2017						 							 *
*                                                                                        *
**PURPOSE:  Creation of data sets of simulation study (no unmeasured confounding)		 *
*																						 *
**WRITE: &path_log (log-File)												       	 	 *
*		 &path_out (simulated data)														 *
*																						 *
**Note: The &path macro variable within the macro needs to be adapted to 			     *
*		to the specific computing environment											 *
*		This program is an adaption from the program published Young et al.		 		 * 
*		https://www.hsph.harvard.edu/causal/software/ (accessed Nov 13th 2017) 			 *
*		Reference: Young, J. G., Hernan, M. A., Picciotto, S., und Robins, J. M. (2010). *
*				   Relation between three classes of structural models for the effect of *
*				   a time-varying exposure on survival. 								 *
*				   Lifetime Data Analysis 16(1): 71-84.									 *
*****************************************************************************************/

options mprint nosymbolgen;


%macro simulate(
	subjects=1000,       /* number of subjects in the simulated data sets */
    tpoints= 10,         /* maximum number of measurements for each subject in the simulated data */         
    psi=0.3,      	     /* scenarios for treatment effect (TE). Possible values 1 -> TE=-0.3; 2 -> TE=0; 3 ->TE=0.3*/
	gamma0 = log(3/7),   /* parameters of the model for confounder L(k)*/
	gamma1 = 2, 
	gamma2 = log(0.5),
	gamma3 = log(1.5),
	alpha0= log(2/7) ,   /*parameters of the model for A(k), remaining parameters are specified below depending on number of confounders*/
	alpha3 = log(4),   
	delta0= -2 , 		 /*parameters of the model for C(k), remaining parameters are specified below depending on number of confounders*/
	delta2 = -0.2,    
	cut=30,				 /*Parameter c*/
	lam=0.01,			 /*scale (lam) and shape (shape) parameter for weibull distribution of counterfactual time-to-event*/
	shape=1,			
	dataout= Simdata_all,/*temporal name of simulated sas data set in SAS work*/
	sim_nr=,			 /*number of simulation*/	
	scen=,				 /*Scenario for number of confounders. Possible values 1 -> 1 conf.; 2-> 5 conf.; 3-> 20 conf.*/
	ever=				 /*Treatment scenario. Possible values 1-> felxible-treatment; 2-> treatment-initiation */
	);

	%*specify path for log and data output;
	%let path_log=%str(I:\SASTOOLS\dats\SIM_&subjects._psi&psi._&scen._ever&ever.\log.log);

	%*write log;
	filename logfile "&path_log";
	proc printto log=logfile new;
	run;

	%*write treatment effect in macro variable psi1 as specified in Table 2;
	%if &psi=1 %then %let psi1=-0.3;
	%else %if &psi=2 %then %let psi1=0;
	%else %let psi1=0.3;

	%*specify remaining paramaters for models for A(k) and C(k) depending on number of confounders as specified in Table 2;
	%if &scen=1 %then %do;
		%let n_cov=1;
		%let alpha11 = 0.5;
		%let alpha21 = 0.5;
		%let delta11 = 0.4;
	%end;
	%else %if &scen=2 %then %do;
		%let n_cov=5;
		%let alpha11 = 0.5;
		%let alpha12 = 0.25;
		%let alpha13 = 0.25;
		%let alpha14 = -0.25;
		%let alpha15 = -0.25;

		%let alpha21 = 0.5;
		%let alpha22 = 0.25;
		%let alpha23 = 0.25;
		%let alpha24 = -0.25;
		%let alpha25 = -0.25;

		%let delta11 = 0.4;
		%let delta12 = 0.4;
		%let delta13 = 0.4;
		%let delta14 = -0.4;
		%let delta15 = -0.4;
	%end;

	%else %do;
		%let n_cov=20;
		%let alpha11 = 0.5;
		%let alpha12 = 0.25;
		%let alpha13 = 0.25;
		%let alpha14 = 0.25;
		%let alpha15 = 0.25;
		%let alpha16 = 0.25;
		%let alpha17 = 0.25;
		%let alpha18 = 0.25;
		%let alpha19 = 0.25;
		%let alpha110 = 0.25;
		%let alpha111 = -0.25;
		%let alpha112 = -0.25;
		%let alpha113 = -0.25;
		%let alpha114 = -0.25;
		%let alpha115 = -0.25;
		%let alpha116 = -0.25;
		%let alpha117 = -0.25;
		%let alpha118 = -0.25;
		%let alpha119 = -0.25;
		%let alpha120 = -0.25;

		%let alpha21 = 0.5;
		%let alpha22 = 0.25;
		%let alpha23 = 0.25;
		%let alpha24 = 0.25;
		%let alpha25 = 0.25;
		%let alpha26 = 0.25;
		%let alpha27 = 0.25;
		%let alpha28 = 0.25;
		%let alpha29 = 0.25;
		%let alpha210 = 0.25;
		%let alpha211 = -0.25;
		%let alpha212 = -0.25;
		%let alpha213 = -0.25;
		%let alpha214 = -0.25;
		%let alpha215 = -0.25;
		%let alpha216 = -0.25;
		%let alpha217 = -0.25;
		%let alpha218 = -0.25;
		%let alpha219 = -0.25;
		%let alpha220 = -0.25;

		%let delta11 = 0.4;
		%let delta12 = 0.4;
		%let delta13 = 0.4;
		%let delta14 = 0.4;
		%let delta15 = 0.4;
		%let delta16 = 0.4;
		%let delta17 = 0.4;
		%let delta18 = 0.4;
		%let delta19 = 0.4;
		%let delta110 = 0.4;
		%let delta111 = -0.4;
		%let delta112 = -0.4;
		%let delta113 = -0.4;
		%let delta114 = -0.4;
		%let delta115 = -0.4;
		%let delta116 = -0.4;
		%let delta117 = -0.4;
		%let delta118 = -0.4;
		%let delta119 = -0.4;
		%let delta120 = -0.4;
	%end;


	%*start simulation;
	%do sim=1 %to &sim_nr;

		%*specify path for data output;
		%let path_out=%str(I:\SASTOOLS\dats\SIM_&subjects._psi&psi._&scen._ever&ever.\sim_&sim..csv);

		data &dataout;
			%*initiate variables: A, Am =  Treatment at time k and k-1;
			%*L_&i = Confounder at k;
			%*Y, Y0 = Event indicator at time k and k-1;
			%*maxT = maximum number of time-point (&tpoints);
	    	length  A Am %do i=1 %to &n_cov; L&i. %end; Y Ym  maxT 3 ;
				%*initiate random seeds for simulation;
				sT0 = %sysevalf(&sT0) ;
	            sL = %sysevalf(&sL) ;
	            sA = %sysevalf(&sA) ;
	            sY = %sysevalf(&sY);
	            sC = %sysevalf(&sC);
				%*simulate longitudinal data for each subject ;
	            do id=1 to %sysevalf(&subjects);
				   %*simulate counterfactual time-to-event T_0;
				   call ranexp(sT0,expvar); 
	               T0=(expvar/(&lam**&shape))**(1/&shape); 
				   %*Indicator, if T_0<c;
				   if T0<&cut then IT0=1;
				   else IT0=0;
				   %*temporary variables; 
				   maxT=&tpoints;
	               initA = .;
	               C1time = .; 
	               C2time = . ;
				   %*define arrays of confounders, treatments, evetns, censoring and its probabilities for better adressing;
				   %do i=1 %to &n_cov;
	               	  array Larray&i. L&i._1-L&i._&tpoints;
				   %end;
	               array Aarray A1-A&tpoints;
	               array Yarray Y1-Y&tpoints;
				   array Ymarray Ym1-Ym&tpoints;
				   array Carray C1-C&tpoints;
				   array Cmarray Cm1-Cm&tpoints;
				   array pAarray pA1-pA&tpoints;
	               array pCarray pC1-pC&tpoints;
				   array Marray M1-M&tpoints;
				   %do i=1 %to &n_cov;
	                  array pLarray&i. pL&i._1-pL&i._&tpoints;
				   %end;
				   array chkarray chk1-chk&tpoints;
				   array tarray t1-t&tpoints;
				   %*initiate Y(0) and C(0);
				   j = 1;
	               Ym1=0;
				   Cm1=0;
				   M1=0;
				   %*simulate L(1) according to Table 2 of manuscript;
				   logitL = &gamma0 + &gamma1 * IT0 ;
				   %do i=1 %to &n_cov;
				   	  pL = 1/(1+(1/exp(logitL)));
	                  pLarray&i.(1) = pL;
				   if pL=1 then do; 
	                  Larray&i.(1) = 1;
	               end;
	               else do;                      
	                  call ranbin(sL,1,pL,Larray&i.(1));                 
	               end;
				   %end; 
   				   %*simulate A(1) according to Table 2 of manuscript;
	               logitA = &alpha0 %do i=1 %to &n_cov; + &&alpha1&i * L&i._1 %end;;                    
	               pA1 = 1/(1+(1/exp(logitA)));
				   call ranbin(sA,1,pA1,A1);                      
	               %*generate event indicator according to Table 2 of manuscript;
				   chk1 = exp(&psi1*A1);
	               if T0 > chk1 then do;
					  Y1=0;
					  t1=.;
				   end;
				   else if T0 <= chk1 then do;
					  Y1=1;
					  t1=T0*exp(-&psi1*A1);
				   end;
				   %*simulate censoring indicator according to Table 2 of manuscript;
				   logitC= &delta0;
				   pC1 =  1/(1+(1/exp(logitC)));
				   call ranbin(sC,1,pC1,C1);
   				   %*repeat calculation for remaining time points according to Table 2;
	               do j=2 to &tpoints;
	                  time = j;
	                  if j = C2time + 1 then leave; 
	                  if j = C1time + 1 then leave;  
					  Ymarray(j)=Yarray(j-1);
					  %do i=1 %to &n_cov;
					  logitL = &gamma0 + &gamma1 * IT0 + &gamma2 * Aarray(j-1) + &gamma3 * Larray&i.(j-1);
					  pL=exp(logitL)/(1+exp(logitL));
	                  pLarray&i.(j) = pL;
	                  if pL = 1 then do; 
	                     Larray&i.(j) = 1;
	                  end;
	                  else do;                          
	                     call ranbin(sL,1,pL,Larray&i.(j));                            
	                  end;
					  %end;
					  %*in case of "treatment initiation", once treated (A(j)=1), the patient remains treated;
					  %*Marker M indicates time point when patient is first treated;
					  %if &ever=1 %then %do;
					  	if Aarray(j-1)=1 then do;
							Aarray(j)=1;
						    Marray(j)=Marray(j-1)+1;
						end;
						else do;
						  logitA = &alpha0 %do i=1 %to &n_cov; + &&alpha1&i. * Larray&i.(j) + &&alpha2&i. * Larray&i.(j-1) %end; + &alpha3 * Aarray(j-1);
		                  pAarray(j)=exp(logitA)/(1+exp(logitA));	                                            
		                  call ranbin(sA,1,pAarray(j),Aarray(j)); 
     					  Marray(j)=Marray(j-1);
						end; 
					  %end;
					  %else %do;
		                  logitA = &alpha0 %do i=1 %to &n_cov; + &&alpha1&i. * Larray&i.(j) + &&alpha2&i. * Larray&i.(j-1) %end; + &alpha3 * Aarray(j-1);
		                  pAarray(j)=exp(logitA)/(1+exp(logitA));	                                            
		                  call ranbin(sA,1,pAarray(j),Aarray(j));  
					  %end; 
	                  if Aarray(j)=1 then timeA=j;
	                  if Ymarray(j)=1 then do;
						 Aarray(j)=0;
						 %do i=1 %to &n_cov;
						 	Larray&i.(j)=0;
						 %end;
					  end;
					  chkarray(j) = chkarray(j-1) + exp(&psi1*Aarray(j));
	                  if T0 > chkarray(j) then do;
						 Yarray(j)=0;
						 tarray(j)=.;
					  end;
					  else if T0 <= chkarray(j) then do;
					     Yarray(j)=1;
						 if Ymarray(j)=1 then do;
						    tarray(j)=tarray(j-1);
						 end;
					     else do;
						    tarray(j)=(j-1) + ((T0-chkarray(j-1))*exp(-&psi1*Aarray(j)));
					     end;
					  end;
					  %*if patient is censored at time j-1, he/she remains censored at j;
					  if Carray(j-1)=1 then Carray(j)=1;
					  else do; 
						  logitC= &delta0 %do i=1 %to &n_cov; + &&delta1&i * Larray&i.(j-1) %end; + &delta2 * Aarray(j-1);;
						  pCarray(j) =  exp(logitC)/(1+exp(logitC));
						  call ranbin(sC,1,pCarray(j),Carray(j));
					  end;
					   Cmarray(j)=Carray(j-1);
				   %*end simulation for time points;
				   end; 

 					%*Fill in variables from arrays for later analysis;
	               do tpoint=1 to  &tpoints; 
					  tpoint2 = tpoint-1;
	 				  %do i=1 %to &n_cov;
	                  	L&i.=Larray&i.(tpoint);
	                  	pL&i. = pLarray&i.(tpoint);
					  %end; 
					  chk=chkarray(tpoint);  
	                  A=Aarray(tpoint);
	                  _pA_t=pAarray(tpoint);                                                      
	                  if tpoint=1 then Am=0; else Am=Aarray(tpoint-1);
					  %do i=1 %to &n_cov;
						    if tpoint=1 then Lm&i.=0; else do;
							Lm&i.=Larray&i.(tpoint-1);
							end;
					  %end;
	                  Ym = Ymarray(tpoint);
	                  Y = Yarray(tpoint) ;
	                  C = Carray(tpoint) ;
					  Cm= Cmarray(tpoint);
					  pc= pCarray(tpoint);
					  %* in case of flexible-treatment scenario, marker variable is missing;
					  if tpoint=&tpoints then marker=Marray(tpoint)+A;
					  else marker=Marray(tpoint+1);
	                  output;
	                  end;
	           end;
	           keep id  tpoint  tpoint2 T0 IT0 chk Y Ym C Cm pc A Am %do i=1 %to &n_cov; L&i. Lm&i. %end; marker;
			   %*;
	           call symput('sT0',sT0);
	           call symput('sA',sA);
	           call symput('sL',sL);
	           call symput('sY',sY);
	           call symput('sC',sC);
	       run;

	   %*create output;
	   data &dataout;
   		  retain id tpoint T0 IT0 L1- L&n_cov. Lm1 - Lm&n_cov. A Am Y Ym C Cm marker;
		  set &dataout;
		  %*keep only observations until first event or until censoring;
		  if Ym=0 and Cm=0;
		  keep id tpoint T0 IT0 L1- L&n_cov. Lm1 - Lm&n_cov. A Am Y Ym C Cm marker;
	   run;

	%*export as csv-file;
	proc export data=&dataout
		outfile="&path_out"
   		dbms=csv
		replace;
	run;

%end;
	%*redirect log to SAS window;
	proc printto;
	run;
%mend simulate;


%*initial seed values;
%let sT0 = 123;
%let sL =  1217607105;
%let sA =  1920525422;
%let sY = 2;
%let sC = 987654321;
 
%simulate(sim_nr=1000, subjects=500, psi=1, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=500, psi=1, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=500, psi=1, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=500, psi=1, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=500, psi=1, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=500, psi=1, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=2500, psi=1, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=2500, psi=1, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=2500, psi=1, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=2500, psi=1, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=2500, psi=1, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=2500, psi=1, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=10000, psi=1, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=10000, psi=1, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=10000, psi=1, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=10000, psi=1, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=10000, psi=1, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=10000, psi=1, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=500, psi=2, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=500, psi=2, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=500, psi=2, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=500, psi=2, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=500, psi=2, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=500, psi=2, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=2500, psi=2, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=2500, psi=2, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=2500, psi=2, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=2500, psi=2, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=2500, psi=2, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=2500, psi=2, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=10000, psi=2, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=10000, psi=2, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=10000, psi=2, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=10000, psi=2, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=10000, psi=2, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=10000, psi=2, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=500, psi=3, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=500, psi=3, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=500, psi=3, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=500, psi=3, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=500, psi=3, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=500, psi=3, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=2500, psi=3, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=2500, psi=3, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=2500, psi=3, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=2500, psi=3, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=2500, psi=3, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=2500, psi=3, scen=3, ever=1);

%simulate(sim_nr=1000, subjects=10000, psi=3, scen=1, ever=0);
%simulate(sim_nr=1000, subjects=10000, psi=3, scen=2, ever=0);
%simulate(sim_nr=1000, subjects=10000, psi=3, scen=3, ever=0);

%simulate(sim_nr=1000, subjects=10000, psi=3, scen=1, ever=1);
%simulate(sim_nr=1000, subjects=10000, psi=3, scen=2, ever=1);
%simulate(sim_nr=1000, subjects=10000, psi=3, scen=3, ever=1);
