#################################################################################################
#                                                                                               #
##PROJECT: Robust versus consistent variance estimators in marginal					                    #
#		    	 structural Cox models									                                              #
#                                                                                               #
##PROGRAM NAME: 3_sim_ana1.R      	                                                            #
#                                                                                               #
##AUTHOR:       D. Enders					                                     	                        #
#                                                                                               #
##VERSION/DATE: V 1.0 / Dec 22nd 2017   				 								                                #
#                                                                                               #
##PURPOSE:      MSM analysisis with adpated and robust variance formula                         #
#               Scenario of no unmeasured confounding                                           #
#                                                                                               #
##Read: path1 - path to MSM function ana_msm with flexible-treatment modeling                   #
#       path2 - path to MSM function ana_msm2 with treatment-initiation modeling                #
#       path3 - path to input data sets (created by SAS function 1_sim_data1.sas)               #
#       path4 - path to output data sets (for post processing in 5_graphs.r)                    #
#################################################################################################

#########################
#Options  and Packages  #
#########################

memory.limit(size=8000000)
library(PermAlgo)
library(survival)
library(geepack)
library(boot)
library(ipw)
library(Matrix)
library(simex)
library(turner)
library(plyr)
library(lgcp)

#path to function conducting MSM analysis with flexible-treatment
path1<-"G:/2-PhaseTransfer/Ablage/Publikationen/Varianzschtzung MSMs/PROGS/ana_msm.r" 
#path to function conducting MSM analysis with treatment-initiation
path2<-"G:/2-PhaseTransfer/Ablage/Publikationen/Varianzschtzung MSMs/PROGS/ana_msm2.r" 

#### load analysis functions for adapted msm analysis
source(path1);
source(path2);

#number of simulations
simnr<-1000
################
#Functions     #
################
ana<-function(scen, exp, subjects, ncov, ever)
{

  for (sim in 1:simnr)
  {
    #path to input data for analyses
    path3<-paste("I:/SASTOOLS/dats/SIM_",subjects,"_psi",exp,"_",scen,"_ever",ever,"/sim_",sim,".csv",sep="")
    #path to results
    path4<-paste("I:/SASTOOLS/outputs/results_",subjects,"_psi",exp,"_",scen,"_ever",ever,".csv",sep="")
    
    if (sim%%10==0)
    {
      #keep track of simulations
      print(paste(sim," Simulations ",Sys.time()),sep="")
    }
    #concatenate names of confounders for later modeling
    #for more than 1 covariate
    if (ncov>1)
    {
      Lname<-"L1"
      Lmod<-"L1"
      for (i in 2:ncov)
      {
        Lname<-c(Lname,paste("L",i,sep=""))
        Lmod<-paste(Lmod,"+L",i,sep="")
      }
      Lmname<-"Lm1"
      Lmod<-paste(Lmod,"+Lm1",sep="")
      Lmod.c<-"Lm1"
      for (i in 2:ncov)
      {
        Lmname<-c(Lmname,paste("Lm",i,sep=""))
        Lmod<-paste(Lmod,"+Lm",i,sep="")
        Lmod.c<-paste(Lmod.c,"+Lm",i,sep="")
      }
    }
    #for 1 confounder
    else
    {
      Lname<-"L1"
      Lmod<-"L1"
      Lmname<-"Lm1"
      Lmod<-paste(Lmod,"+Lm1",sep="")
      Lmod.c<-"Lm1"
    }
      #vector of column names
      nam<-c("Id","tp","T0","IT0",Lname,Lmname,"A","Am","Y","Ym","C","Cm","marker")
      #read simulated data set
      out<-read.csv(path3)
      #assign column names
      colnames(out)<-nam
      #prepare for msm analysis, i.e. build formulas and list of covariates
      form.denom<-paste("A~",Lmod,"+Am",sep="")
      form.num<-paste("A~Am")
      list.denom<-c(Lname,Lmname,"Am")
      list.num<-c("Am")
      form.denom.cens<-paste("C~",Lmod.c,"+Am",sep="")
      form.num.cens<-paste("C~Am")
      list.denom.cens<-c(Lmname,"Am")
      list.num.cens<-c("Am")
      #conduct msm analysis with consistent and robust variance estimator
      if (ever==0)
      {
        results<-msm.ana(dat=out,
                          ID=Id,
                          cens=1,
                          C=1,
                          stab=1,
                          form.denom=form.denom,
                          form.num=form.num,
                          form.denom.cens=form.denom.cens,
                          form.num.cens=form.num.cens,
                          list.denom=list.denom,
                          list.num=list.num,
                          list.denom.cens=list.denom.cens,
                          list.num.cens=list.num.cens
                          )
      }
      else
      {
        #in treatment-initiation scenario, prior treatment (A(k-1)) must be excluded 
        #from treatment model for A(k) since it is not informative,
        form.denom<-paste("A~",Lmod,sep="")
        form.num<-paste("A~1")
        list.denom<-c(Lname,Lmname)
        list.num<-c("INTERCEPT")
        results<-msm.ana2(dat=out,
                         ID=Id,
                         cens=1,
                         C=1,
                         stab=1,
                         form.denom=form.denom,
                         form.num=form.num,
                         form.denom.cens=form.denom.cens,
                         form.num.cens=form.num.cens,
                         list.denom=list.denom,
                         list.num=list.num,
                         list.denom.cens=list.denom.cens,
                         list.num.cens=list.num.cens,
                         marker=marker
        )
      }
      #combine all results
      if (sim==1)
      {
        results.all<-results
      }
      else
      {
        results.all<-rbind(results.all,results)
      }
  }
  #write to csv
  results<-as.data.frame(results.all)
  colnames(results)<-c("TREAT","VAR1","VAR2")
  write.csv(results,file=path4)
  return(results)
}

###conduct analyses for simulated data
results_500_psi1_1_ever0<-ana(scen=1, exp=1, subjects=500, ncov=1, ever=0)
results_500_psi1_2_ever0<-ana(scen=2, exp=1, subjects=500, ncov=5, ever=0)
results_500_psi1_3_ever0<-ana(scen=3, exp=1, subjects=500, ncov=20, ever=0)

results_500_psi2_1_ever0<-ana(scen=1, exp=2, subjects=500, ncov=1, ever=0)
results_500_psi2_2_ever0<-ana(scen=2, exp=2, subjects=500, ncov=5, ever=0)
results_500_psi2_3_ever0<-ana(scen=3, exp=2, subjects=500, ncov=20, ever=0)

results_500_psi3_1_ever0<-ana(scen=1, exp=3, subjects=500, ncov=1, ever=0)
results_500_psi3_2_ever0<-ana(scen=2, exp=3, subjects=500, ncov=5, ever=0)
results_500_psi3_3_ever0<-ana(scen=3, exp=3, subjects=500, ncov=20, ever=0)

results_500_psi1_1_ever1<-ana(scen=1, exp=1, subjects=500, ncov=1, ever=1)
results_500_psi1_2_ever1<-ana(scen=2, exp=1, subjects=500, ncov=5, ever=1)
results_500_psi1_3_ever1<-ana(scen=3, exp=1, subjects=500, ncov=20, ever=1)

results_500_psi2_1_ever1<-ana(scen=1, exp=2, subjects=500, ncov=1, ever=1)
results_500_psi2_2_ever1<-ana(scen=2, exp=2, subjects=500, ncov=5, ever=1)
results_500_psi2_3_ever1<-ana(scen=3, exp=2, subjects=500, ncov=20, ever=1)

results_500_psi3_1_ever1<-ana(scen=1, exp=3, subjects=500, ncov=1, ever=1)
results_500_psi3_2_ever1<-ana(scen=2, exp=3, subjects=500, ncov=5, ever=1)
results_500_psi3_3_ever1<-ana(scen=3, exp=3, subjects=500, ncov=20, ever=1)

results_2500_psi1_1_ever0<-ana(scen=1, exp=1, subjects=2500, ncov=1, ever=0)
results_2500_psi1_2_ever0<-ana(scen=2, exp=1, subjects=2500, ncov=5, ever=0)
results_2500_psi1_3_ever0<-ana(scen=3, exp=1, subjects=2500, ncov=20, ever=0)

results_2500_psi2_1_ever0<-ana(scen=1, exp=2, subjects=2500, ncov=1, ever=0)
results_2500_psi2_2_ever0<-ana(scen=2, exp=2, subjects=2500, ncov=5, ever=0)
results_2500_psi2_3_ever0<-ana(scen=3, exp=2, subjects=2500, ncov=20, ever=0)

results_2500_psi3_1_ever0<-ana(scen=1, exp=3, subjects=2500, ncov=1, ever=0)
results_2500_psi3_2_ever0<-ana(scen=2, exp=3, subjects=2500, ncov=5, ever=0)
results_2500_psi3_3_ever0<-ana(scen=3, exp=3, subjects=2500, ncov=20, ever=0)

results_2500_psi1_1_ever1<-ana(scen=1, exp=1, subjects=2500, ncov=1, ever=1)
results_2500_psi1_2_ever1<-ana(scen=2, exp=1, subjects=2500, ncov=5, ever=1)
results_2500_psi1_3_ever1<-ana(scen=3, exp=1, subjects=2500, ncov=20, ever=1)

results_2500_psi2_1_ever1<-ana(scen=1, exp=2, subjects=2500, ncov=1, ever=1)
results_2500_psi2_2_ever1<-ana(scen=2, exp=2, subjects=2500, ncov=5, ever=1)
results_2500_psi2_3_ever1<-ana(scen=3, exp=2, subjects=2500, ncov=20, ever=1)

results_2500_psi3_1_ever1<-ana(scen=1, exp=3, subjects=2500, ncov=1, ever=1)
results_2500_psi3_2_ever1<-ana(scen=2, exp=3, subjects=2500, ncov=5, ever=1)
results_2500_psi3_3_ever1<-ana(scen=3, exp=3, subjects=2500, ncov=20, ever=1)

results_10000_psi1_1_ever0<-ana(scen=1, exp=1, subjects=10000, ncov=1, ever=0)
results_10000_psi1_2_ever0<-ana(scen=2, exp=1, subjects=10000, ncov=5, ever=0)
results_10000_psi1_3_ever0<-ana(scen=3, exp=1, subjects=10000, ncov=20, ever=0)

results_10000_psi2_1_ever0<-ana(scen=1, exp=2, subjects=10000, ncov=1, ever=0)
results_10000_psi2_2_ever0<-ana(scen=2, exp=2, subjects=10000, ncov=5, ever=0)
results_10000_psi2_3_ever0<-ana(scen=3, exp=2, subjects=10000, ncov=20, ever=0)

results_10000_psi3_1_ever0<-ana(scen=1, exp=3, subjects=10000, ncov=1, ever=0)
results_10000_psi3_2_ever0<-ana(scen=2, exp=3, subjects=10000, ncov=5, ever=0)
results_10000_psi3_3_ever0<-ana(scen=3, exp=3, subjects=10000, ncov=20, ever=0)

results_10000_psi1_1_ever1<-ana(scen=1, exp=1, subjects=10000, ncov=1, ever=1)
results_10000_psi1_2_ever1<-ana(scen=2, exp=1, subjects=10000, ncov=5, ever=1)
results_10000_psi1_3_ever1<-ana(scen=3, exp=1, subjects=10000, ncov=20, ever=1)

results_10000_psi2_1_ever1<-ana(scen=1, exp=2, subjects=10000, ncov=1, ever=1)
results_10000_psi2_2_ever1<-ana(scen=2, exp=2, subjects=10000, ncov=5, ever=1)
results_10000_psi2_3_ever1<-ana(scen=3, exp=2, subjects=10000, ncov=20, ever=1)

results_10000_psi3_1_ever1<-ana(scen=1, exp=3, subjects=10000, ncov=1, ever=1)
results_10000_psi3_2_ever1<-ana(scen=2, exp=3, subjects=10000, ncov=5, ever=1)
results_10000_psi3_3_ever1<-ana(scen=3, exp=3, subjects=10000, ncov=20, ever=1)


